@@ -1,36 +1,37 @@
 import os
 import sys
 import time
+
 from PyQt5.QtWidgets import (
     QDialog, QDialogButtonBox, QTreeWidgetItem,
-    QCompleter, QMessageBox, QFileDialog, QWidget)
-from PyQt5.QtGui import QIcon, QPixmap
+    QCompleter, QMessageBox, QFileDialog)
+from PyQt5.QtGui import QIcon, QPixmap, QGuiApplication
 from PyQt5.QtCore import Qt, QTimer
 
 import ray
 from gui_server_thread import GUIServerThread
-from gui_tools import (default_session_root, ErrDaemon, _translate,
+from gui_tools import (ErrDaemon, _translate,
                        CommandLineArgs, RS, isDarkTheme)
 
 import ui_open_session
 import ui_new_session
-import ui_list_snapshots
 import ui_save_template_session
 import ui_nsm_open_info
 import ui_abort_session
 import ui_about_raysession
-import ui_add_application
 import ui_donations
+import ui_jack_config_info
 import ui_new_executable
 import ui_error_dialog
 import ui_quit_app
-import ui_client_properties
+import ui_script_info
+import ui_script_user_action
+import ui_session_scripts_info
 import ui_stop_client
 import ui_stop_client_no_save
 import ui_abort_copy
 import ui_client_trash
 import ui_daemon_url
-import ui_edit_executable
 import ui_snapshot_progress
 import ui_waiting_close_user
 
@@ -40,15 +41,15 @@ class ChildDialog(QDialog):
         self._session = parent._session
         self._signaler = self._session._signaler
 
-        daemon_manager = self._session._daemon_manager
-        self.daemon_launched_before = daemon_manager.launched_before
+        self._daemon_manager = self._session._daemon_manager
 
         self._signaler.server_status_changed.connect(self.serverStatusChanged)
         self._signaler.server_copying.connect(self.serverCopying)
 
         self.server_copying = parent.server_copying
-
-    def toDaemon(self, *args):
+    
+    @classmethod
+    def toDaemon(cls, *args):
         server = GUIServerThread.instance()
         if server:
             server.toDaemon(*args)
@@ -57,47 +58,47 @@ class ChildDialog(QDialog):
                              % args)
 
     def serverStatusChanged(self, server_status):
-        return
+        pass
 
     def serverCopying(self, bool_copying):
         self.server_copying = bool_copying
         self.serverStatusChanged(self._session.server_status)
-        
+
     def changeRootFolder(self):
         root_folder = QFileDialog.getExistingDirectory(
-            self, 
+            self,
             _translate("root_folder_dialogs",
-                       "Choose root folder for sessions"), 
-            CommandLineArgs.session_root, 
+                       "Choose root folder for sessions"),
+            CommandLineArgs.session_root,
             QFileDialog.ShowDirsOnly)
-        
+
         if not root_folder:
             return
-        
+
         # Security, kde dialogs sends $HOME if user type a folder path
         # that doesn't already exists.
         if os.getenv('HOME') and root_folder == os.getenv('HOME'):
             return
-        
+
         errorDialog = QMessageBox(
-            QMessageBox.Critical, 
-            _translate('root_folder_dialogs', 'unwritable dir'), 
+            QMessageBox.Critical,
+            _translate('root_folder_dialogs', 'unwritable dir'),
             _translate(
                 'root_folder_dialogs',
                 '<p>You have no permissions for %s,<br>' % root_folder \
                     + 'choose another directory !</p>'))
-        
+
         if not os.path.exists(root_folder):
             try:
                 os.makedirs(root_folder)
             except:
                 errorDialog.exec()
                 return
-        
+
         if not os.access(root_folder, os.W_OK):
             errorDialog.exec()
             return
-        
+
         RS.settings.setValue('default_session_root', root_folder)
         self.toDaemon('/ray/server/change_root', root_folder)
 
@@ -112,73 +113,73 @@ class ChildDialog(QDialog):
 
 
 class SessionItem(QTreeWidgetItem):
-    def __init__(self, list):
-        QTreeWidgetItem.__init__(self, list)
-        
+    def __init__(self, l_list):
+        QTreeWidgetItem.__init__(self, l_list)
+
     def showConditionnaly(self, string):
         show = bool(string.lower() in self.data(0, Qt.UserRole).lower())
-        
-        n=0
+
+        n = 0
         for i in range(self.childCount()):
             if self.child(i).showConditionnaly(string.lower()):
-                n+=1
+                n += 1
         if n:
             show = True
-            
+
         self.setExpanded(bool(n and string))
         self.setHidden(not show)
         return show
-    
+
     def findItemWith(self, string):
         if self.data(0, Qt.UserRole) == string:
             return self
-        
+
         item = None
-        
+
         for i in range(self.childCount()):
             item = self.child(i).findItemWith(string)
             if item:
                 break
-            
+
         return item
-        
+
     def __lt__(self, other):
         if self.childCount() and not other.childCount():
             return True
-        
+
         if other.childCount() and not self.childCount():
             return False
-        
+
         return bool(self.text(0).lower() < other.text(0).lower())
 
 class SessionFolder:
     name = ""
     has_session = True
     path = ""
-    
+
     def __init__(self, name):
         self.name = name
         self.subfolders = []
-    
+
     def setPath(self, path):
         self.path = path
-    
+
     def makeItem(self):
         item = SessionItem([self.name])
-        
+
         item.setData(0, Qt.UserRole, self.path)
         if self.subfolders:
             item.setIcon(0, QIcon.fromTheme('folder'))
-        
+
         if not self.path:
             item.setFlags(item.flags() & ~Qt.ItemIsSelectable)
-        
+
         for folder in self.subfolders:
             sub_item = folder.makeItem()
             item.addChild(sub_item)
-            
+
         return item
-    
+
 class OpenSessionDialog(ChildDialog):
     def __init__(self, parent):
         ChildDialog.__init__(self, parent)
@@ -195,13 +196,13 @@ class OpenSessionDialog(ChildDialog):
         self.ui.filterBar.updownpressed.connect(self.updownPressed)
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
         self.ui.currentSessionsFolder.setText(CommandLineArgs.session_root)
-        
+
         self._signaler.add_sessions_to_list.connect(self.addSessions)
         self._signaler.root_changed.connect(self.rootChanged)
 
         self.toDaemon('/ray/server/list_sessions', 0)
 
-        if self.daemon_launched_before:
+        if not self._daemon_manager.is_local:
             self.ui.toolButtonFolder.setVisible(False)
             self.ui.currentSessionsFolder.setVisible(False)
             self.ui.labelSessionsFolder.setVisible(False)
@@ -210,12 +211,11 @@ class OpenSessionDialog(ChildDialog):
         self.has_selection = False
 
         self.serverStatusChanged(self._session.server_status)
-        
+
         self.folders = []
         self.all_items = []
-        
+
         self._last_mouse_click = 0
-        
 
     def serverStatusChanged(self, server_status):
         self.ui.toolButtonFolder.setEnabled(
@@ -228,7 +228,7 @@ class OpenSessionDialog(ChildDialog):
                 ray.ServerStatus.OFF,
                 ray.ServerStatus.READY) and not self.server_copying)
         self.preventOk()
-        
+
     def rootChanged(self, session_root):
         self.ui.currentSessionsFolder.setText(session_root)
         self.ui.sessionList.clear()
@@ -239,14 +239,14 @@ class OpenSessionDialog(ChildDialog):
         for session_name in session_names:
             folder_div = session_name.split('/')
             folders = self.folders
-            
+
             for i in range(len(folder_div)):
                 f = folder_div[i]
                 for g in folders:
                     if g.name == f:
                         if i+1 == len(folder_div):
                             g.setPath(session_name)
-                            
+
                         folders = g.subfolders
                         break
                 else:
@@ -255,22 +255,22 @@ class OpenSessionDialog(ChildDialog):
                         new_folder.setPath(session_name)
                     folders.append(new_folder)
                     folders = new_folder.subfolders
-                        
+
         self.ui.sessionList.clear()
-        
+
         for folder in self.folders:
             item = folder.makeItem()
             self.ui.sessionList.addTopLevelItem(item)
-        
+
         self.ui.sessionList.sortByColumn(0, Qt.AscendingOrder)
-        
+
         # Try to select last used session
         root_item = self.ui.sessionList.invisibleRootItem()
         for i in range(root_item.childCount()):
             item = root_item.child(i)
             last_session_item = item.findItemWith(
                 RS.settings.value('last_session', type=str))
-            
+
             if last_session_item:
                 self.ui.sessionList.setCurrentItem(last_session_item)
                 self.ui.sessionList.scrollToItem(last_session_item)
@@ -305,7 +305,7 @@ class OpenSessionDialog(ChildDialog):
     def updownPressed(self, key):
         root_item = self.ui.sessionList.invisibleRootItem()
         row = self.ui.sessionList.currentIndex().row()
-        
+
         if key == Qt.Key_Up:
             if row == 0:
                 return
@@ -322,7 +322,7 @@ class OpenSessionDialog(ChildDialog):
                 if row == root_item.childCount() - 1:
                     return
                 row += 1
-                
+
         self.ui.sessionList.setCurrentItem(root_item.child(row))
 
     def currentItemChanged(self, item, previous_item):
@@ -336,24 +336,24 @@ class OpenSessionDialog(ChildDialog):
     def getSelectedSession(self):
         if self.ui.sessionList.currentItem():
             return self.ui.sessionList.currentItem().data(0, Qt.UserRole)
-    
+
     def deployItem(self, item, column):
         if not item.childCount():
-            return 
-        
+            return
+
         if time.time() - self._last_mouse_click > 0.35:
             item.setExpanded(not item.isExpanded())
-            
+
         self._last_mouse_click = time.time()
-        
+
     def goIfAny(self, item, column):
         if item.childCount():
-            return 
-        
+            return
+
         if (self.server_will_accept and self.has_selection
-            and self.ui.sessionList.currentItem().data(0, Qt.UserRole)):
-                self.accept()
-            
+                and self.ui.sessionList.currentItem().data(0, Qt.UserRole)):
+            self.accept()
+
 
 class NewSessionDialog(ChildDialog):
     def __init__(self, parent, duplicate_window=False):
@@ -377,9 +377,9 @@ class NewSessionDialog(ChildDialog):
         self._signaler.add_sessions_to_list.connect(self.addSessionsToList)
         self._signaler.session_template_found.connect(self.addTemplatesToList)
         self._signaler.root_changed.connect(self.rootChanged)
-        
+
         self.toDaemon('/ray/server/list_sessions', 1)
-        
+
         if self.is_duplicate:
             self.ui.labelTemplate.setVisible(False)
             self.ui.comboBoxTemplate.setVisible(False)
@@ -389,20 +389,20 @@ class NewSessionDialog(ChildDialog):
         else:
             self.toDaemon('/ray/server/list_session_templates')
 
-        if self.daemon_launched_before:
+        if not self._daemon_manager.is_local:
             self.ui.toolButtonFolder.setVisible(False)
             self.ui.currentSessionsFolder.setVisible(False)
             self.ui.labelSessionsFolder.setVisible(False)
 
         self.initTemplatesComboBox()
         self.setLastTemplateSelected()
-        
-        self.ui.labelSubFolder.setVisible(False)
-        self.ui.comboBoxSubFolder.setVisible(False)
 
         self.server_will_accept = False
         self.text_is_valid = False
 
+        self.completer = QCompleter(self.sub_folders)
+        self.ui.lineEdit.setCompleter(self.completer)
+
         self.serverStatusChanged(self._session.server_status)
 
     def serverStatusChanged(self, server_status):
@@ -418,41 +418,44 @@ class NewSessionDialog(ChildDialog):
                 server_status == ray.ServerStatus.READY and not self.server_copying)
 
         self.preventOk()
-    
+
     def rootChanged(self, session_root):
         self.ui.currentSessionsFolder.setText(session_root)
         self.session_list.clear()
         self.sub_folders.clear()
-        self.initSubFolderCombobox()
         self.toDaemon('/ray/server/list_sessions', 1)
-    
+
     def initTemplatesComboBox(self):
         self.ui.comboBoxTemplate.clear()
         self.ui.comboBoxTemplate.addItem(
             _translate('session_template', "empty"))
         self.ui.comboBoxTemplate.addItem(
-            _translate(
-                'session_template',
-                "with JACK patch memory"))
-        self.ui.comboBoxTemplate.insertSeparator(2)
+            _translate('session_template', "with JACK patch memory"))
+        self.ui.comboBoxTemplate.addItem(
+            _translate('session_template', "with JACK config memory"))
+        self.ui.comboBoxTemplate.addItem(
+            _translate('session_template', "with basic scripts"))
+
+        misscount = self.ui.comboBoxTemplate.count()  - 1 - len(
+                                                ray.factory_session_templates)
+        for i in range(misscount):
+            self.ui.comboBoxTemplate.addItem(
+                ray.factory_session_templates[-i])
+
+        self.ui.comboBoxTemplate.insertSeparator(
+                                    len(ray.factory_session_templates) + 1)
 
-    def initSubFolderCombobox(self):
-        self.ui.comboBoxSubFolder.clear()
-        self.ui.comboBoxSubFolder.addItem(_translate('new_session', 'none'))
-        
-        for sub_folder in self.sub_folders:
-            self.ui.comboBoxSubFolder.addItem(QIcon.fromTheme('folder'), 
-                                              sub_folder)
-        
-        self.ui.labelSubFolder.setVisible(bool(self.sub_folders))
-        self.ui.comboBoxSubFolder.setVisible(bool(self.sub_folders))
-        
     def setLastTemplateSelected(self):
         last_used_template = RS.settings.value('last_used_template', type=str)
 
         if last_used_template.startswith('///'):
-            if last_used_template == '///withJACKPATCH':
-                self.ui.comboBoxTemplate.setCurrentIndex(1)
+            last_factory_template = last_used_template.replace('///', '', 1)
+
+            for i in range(len(ray.factory_session_templates)):
+                factory_template = ray.factory_session_templates[i]
+                if factory_template == last_factory_template:
+                    self.ui.comboBoxTemplate.setCurrentIndex(i+1)
+                    break
         else:
             if last_used_template in self.template_list:
                 self.ui.comboBoxTemplate.setCurrentText(last_used_template)
@@ -462,23 +465,23 @@ class NewSessionDialog(ChildDialog):
 
     def setLastSubFolderSelected(self):
         last_subfolder = RS.settings.value('last_subfolder', type=str)
-        
-        if last_subfolder:
-            self.ui.comboBoxSubFolder.setCurrentText(last_subfolder)
-        else:
-            self.ui.comboBoxSubFolder.setCurrentIndex(0)
-    
+        if last_subfolder and not self.ui.lineEdit.text():
+            self.ui.lineEdit.setText(last_subfolder + '/')
+
     def addSessionsToList(self, session_names):
         self.session_list += session_names
-        
+
         for session_name in session_names:
             if '/' in session_name:
                 new_dir = os.path.dirname(session_name)
                 if not new_dir in self.sub_folders:
                     self.sub_folders.append(new_dir)
-        
+
         self.sub_folders.sort()
-        self.initSubFolderCombobox()
+        del self.completer
+        self.completer = QCompleter([f + '/' for f in self.sub_folders])
+        self.ui.lineEdit.setCompleter(self.completer)
+
         self.setLastSubFolderSelected()
 
     def addTemplatesToList(self, template_list):
@@ -498,38 +501,24 @@ class NewSessionDialog(ChildDialog):
 
         self.setLastTemplateSelected()
 
-    def getSessionName(self):
-        if self.ui.comboBoxSubFolder.currentIndex() > 0:
-            return '%s/%s' % (self.ui.comboBoxSubFolder.currentText(),
-                              self.ui.lineEdit.text())
-        else:
-            return self.ui.lineEdit.text()
+    def getSessionShortPath(self)->str:
+        return self.ui.lineEdit.text()
 
-    def getTemplateName(self):
-        if self.ui.comboBoxTemplate.currentIndex() == 0:
+    def getTemplateName(self)->str:
+        index = self.ui.comboBoxTemplate.currentIndex()
+
+        if index == 0:
             return ""
 
-        if self.ui.comboBoxTemplate.currentIndex() == 1:
-            return '///withJACKPATCH'
+        if index <= len(ray.factory_session_templates):
+            return '///' + ray.factory_session_templates[index-1]
 
         return self.ui.comboBoxTemplate.currentText()
-    
-    def getSubFolder(self):
-        if self.ui.comboBoxSubFolder.currentIndex() == 0:
-            return ""
-        
-        return  self.ui.comboBoxSubFolder.currentText()
-    
+
     def textChanged(self, text):
-        full_session_text = text
-        if self.ui.comboBoxSubFolder.currentIndex():
-            full_session_text = "%s/%s" % (
-                                    self.ui.comboBoxSubFolder.currentText(),
-                                    text)
-            
         self.text_is_valid = bool(text
-                                  and full_session_text 
-                                        not in self.session_list)
+                                  and not text.endswith('/')
+                                  and text not in self.session_list)
         self.preventOk()
 
     def preventOk(self):
@@ -636,69 +625,6 @@ class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
             self.template_list.append(template.split('/')[0])
 
 
-class EditExecutableDialog(ChildDialog):
-    def __init__(self, parent, client):
-        ChildDialog.__init__(self, parent)
-        self.ui = ui_edit_executable.Ui_Dialog()
-        self.ui.setupUi(self)
-
-        self.ui.lineEditExecutable.setText(client.executable_path)
-        self.ui.lineEditArguments.setText(client.arguments)
-
-    def getExecutable(self):
-        return self.ui.lineEditExecutable.text()
-
-    def getArguments(self):
-        return self.ui.lineEditArguments.text()
-
-
-class ClientPropertiesDialog(ChildDialog):
-    def __init__(self, parent, client):
-        ChildDialog.__init__(self, parent)
-        self.ui = ui_client_properties.Ui_Dialog()
-        self.ui.setupUi(self)
-
-        self.client = client
-
-        self.ui.lineEditIcon.textEdited.connect(self.changeIconwithText)
-        self.ui.pushButtonSaveChanges.clicked.connect(self.saveChanges)
-        self.ui.toolButtonEditExecutable.clicked.connect(self.editExecutable)
-
-    def updateContents(self):
-        self.ui.labelExecutable.setText(self.client.executable_path)
-        self.ui.labelArguments.setText(self.client.arguments)
-        self.ui.labelId.setText(self.client.client_id)
-        self.ui.labelClientName.setText(self.client.name)
-        self.ui.lineEditIcon.setText(self.client.icon_name)
-        self.ui.lineEditLabel.setText(self.client.label)
-        self.ui.checkBoxSaveStop.setChecked(self.client.check_last_save)
-        self.ui.toolButtonIcon.setIcon(
-            ray.getAppIcon(self.client.icon_name, self))
-        self.ui.lineEditIgnoredExtensions.setText(
-            self.client.ignored_extensions)
-
-    def changeIconwithText(self, text):
-        self.ui.toolButtonIcon.setIcon(ray.getAppIcon(text, self))
-
-    def editExecutable(self):
-        dialog = EditExecutableDialog(self, self.client)
-        dialog.exec()
-        if dialog.result():
-            self.ui.labelExecutable.setText(dialog.getExecutable())
-            self.ui.labelArguments.setText(dialog.getArguments())
-
-    def saveChanges(self):
-        self.client.executable_path = self.ui.labelExecutable.text()
-        self.client.arguments = self.ui.labelArguments.text()
-        self.client.label = self.ui.lineEditLabel.text()
-        self.client.icon_name = self.ui.lineEditIcon.text()
-        self.client.check_last_save = self.ui.checkBoxSaveStop.isChecked()
-        self.client.ignored_extensions = self.ui.lineEditIgnoredExtensions.text()
-        self.client.sendPropertiesToDaemon()
-        # better for user to wait a little before close the window
-        QTimer.singleShot(150, self.accept)
-
-
 class ClientTrashDialog(ChildDialog):
     def __init__(self, parent, client_data):
         ChildDialog.__init__(self, parent)
@@ -727,7 +653,7 @@ class ClientTrashDialog(ChildDialog):
 
     def removeClient(self):
         self.toDaemon(
-            '/ray/trash/remove_definitely',
+            '/ray/trashed_client/remove_definitely',
             self.client_data.client_id)
         self.reject()
 
@@ -803,8 +729,9 @@ class OpenNsmSessionInfoDialog(ChildDialog):
         self.ui.setupUi(self)
 
         self.ui.checkBox.stateChanged.connect(self.showThis)
-
-    def showThis(self, state):
+    
+    @classmethod
+    def showThis(cls, state):
         RS.settings.setValue('OpenNsmSessionInfo', not bool(state))
 
 
@@ -816,6 +743,7 @@ class QuitAppDialog(ChildDialog):
         self.ui.pushButtonCancel.setFocus(Qt.OtherFocusReason)
         self.ui.pushButtonSaveQuit.clicked.connect(self.closeSession)
         self.ui.pushButtonQuitNoSave.clicked.connect(self.abortSession)
+        self.ui.pushButtonDaemon.clicked.connect(self.leaveDaemonRunning)
 
         original_text = self.ui.labelExecutable.text()
         self.ui.labelExecutable.setText(
@@ -841,6 +769,10 @@ class QuitAppDialog(ChildDialog):
     def abortSession(self):
         self.toDaemon('/ray/session/abort')
 
+    def leaveDaemonRunning(self):
+        self._daemon_manager.disannounce()
+        QTimer.singleShot(10, QGuiApplication.quit)
+
 
 class AboutRaySessionDialog(ChildDialog):
     def __init__(self, parent):
@@ -856,22 +788,22 @@ class NewExecutableDialog(ChildDialog):
         ChildDialog.__init__(self, parent)
         self.ui = ui_new_executable.Ui_DialogNewExecutable()
         self.ui.setupUi(self)
-        
+
         self.ui.groupBoxAdvanced.setVisible(False)
         self.resize(0, 0)
         self.ui.labelPrefixMode.setToolTip(
             self.ui.comboBoxPrefixMode.toolTip())
         self.ui.labelClientId.setToolTip(self.ui.lineEditClientId.toolTip())
-        
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
-        self.ui.lineEdit.setFocus(Qt.OtherFocusReason)
-        self.ui.lineEdit.textChanged.connect(self.textChanged)
 
-        self.ui.checkBoxProxy.stateChanged.connect(self.proxyStateChanged)
-        
+        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+
+        self.ui.lineEdit.setFocus(Qt.OtherFocusReason)
+        self.ui.lineEdit.textChanged.connect(self.checkAllow)
+        self.ui.checkBoxNsm.stateChanged.connect(self.checkAllow)
+
         self.ui.lineEditPrefix.setEnabled(False)
         self.ui.toolButtonAdvanced.clicked.connect(self.showAdvanced)
-        
+
         self.ui.comboBoxPrefixMode.addItem(
             _translate('new_executable', 'Custom'))
         self.ui.comboBoxPrefixMode.addItem(
@@ -879,10 +811,10 @@ class NewExecutableDialog(ChildDialog):
         self.ui.comboBoxPrefixMode.addItem(
             _translate('new_executable', 'Session Name'))
         self.ui.comboBoxPrefixMode.setCurrentIndex(2)
-        
+
         self.ui.comboBoxPrefixMode.currentIndexChanged.connect(
             self.prefixModeChanged)
-        
+
         self._signaler.new_executable.connect(self.addExecutableToCompleter)
         self.toDaemon('/ray/server/list_path')
 
@@ -892,18 +824,18 @@ class NewExecutableDialog(ChildDialog):
         self.ui.lineEdit.setCompleter(self.completer)
 
         self.ui.lineEdit.returnPressed.connect(self.closeNow)
-        
+
         self.serverStatusChanged(self._session.server_status)
-        
+
         self.text_will_accept = False
 
     def showAdvanced(self):
         self.ui.groupBoxAdvanced.setVisible(True)
         self.ui.toolButtonAdvanced.setVisible(False)
-    
+
     def prefixModeChanged(self, index):
         self.ui.lineEditPrefix.setEnabled(bool(index == 0))
-    
+
     def addExecutableToCompleter(self, executable_list):
         self.exec_list += executable_list
         self.exec_list.sort()
@@ -915,29 +847,27 @@ class NewExecutableDialog(ChildDialog):
     def getExecutableSelected(self):
         return self.ui.lineEdit.text()
 
-    def runViaProxy(self):
-        return bool(self.ui.checkBoxProxy.isChecked())
-    
     def getSelection(self):
         return (self.ui.lineEdit.text(),
-                self.ui.checkBoxProxy.isChecked(),
+                self.ui.checkBoxStartClient.isChecked(),
+                not self.ui.checkBoxNsm.isChecked(),
                 self.ui.comboBoxPrefixMode.currentIndex(),
                 self.ui.lineEditPrefix.text(),
                 self.ui.lineEditClientId.text())
-    
-    def proxyStateChanged(self, state):
-        self.ui.buttonBox.button(
-            QDialogButtonBox.Ok).setEnabled(
-            state or self.text_will_accept)
 
-    def textChanged(self, text):
-        self.text_will_accept = bool(text)
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
-            self.ui.checkBoxProxy.isChecked() or self.text_will_accept)
+    def isAllowed(self):
+        nsm = self.ui.checkBoxNsm.isChecked()
+        text = self.ui.lineEdit.text()
+        allow = bool(bool(text) and (not nsm
+                                     or text in self.exec_list))
+        return allow
+
+    def checkAllow(self):
+        allow = self.isAllowed()
+        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(allow)
 
     def closeNow(self):
-        if (self.ui.lineEdit.text() in self.exec_list
-                or self.ui.checkBoxProxy.isChecked()):
+        if self.isAllowed():
             self.accept()
 
     def serverStatusChanged(self, server_status):
@@ -967,18 +897,16 @@ class StopClientDialog(ChildDialog):
                 minutes = int((time.time() - self.client.last_save) / 60)
                 text = _translate(
                     'client_stop',
-                    "<strong>%s</strong> seems to has not been saved for %i minute(s).<br />Do you really want to stop it ?") % (self.client.prettierName(),
-                                                                                                                                 minutes)
+                    "<strong>%s</strong> seems to has not been saved for %i minute(s).<br />Do you really want to stop it ?") \
+                        % (self.client.prettierName(), minutes)
 
             self.ui.label.setText(text)
-            
+
             self.client.status_changed.connect(self.serverUpdatesClientStatus)
 
         self.ui.pushButtonSaveStop.clicked.connect(self.saveAndStop)
         self.ui.checkBox.stateChanged.connect(self.checkBoxClicked)
 
-        
-
     def saveAndStop(self):
         self.wait_for_save = True
         self.toDaemon('/ray/client/save', self.client_id)
@@ -1002,7 +930,7 @@ class StopClientNoSaveDialog(ChildDialog):
         ChildDialog.__init__(self, parent)
         self.ui = ui_stop_client_no_save.Ui_Dialog()
         self.ui.setupUi(self)
-        
+
         self.client_id = client_id
         self.client = self._session.getClient(client_id)
 
@@ -1010,15 +938,15 @@ class StopClientNoSaveDialog(ChildDialog):
             text = self.ui.label.text() % self.client.prettierName()
             self.ui.label.setText(text)
             self.client.status_changed.connect(self.serverUpdatesClientStatus)
-        
+
         self.ui.checkBox.stateChanged.connect(self.checkBoxClicked)
         self.ui.pushButtonCancel.setFocus(True)
-        
+
     def serverUpdatesClientStatus(self, status):
         if status in (ray.ClientStatus.STOPPED, ray.ClientStatus.REMOVED):
             self.reject()
             return
-        
+
     def checkBoxClicked(self, state):
         self.client.check_last_save = not bool(state)
         self.client.sendPropertiesToDaemon()
@@ -1030,13 +958,108 @@ class SnapShotProgressDialog(ChildDialog):
         self.ui = ui_snapshot_progress.Ui_Dialog()
         self.ui.setupUi(self)
         self._signaler.server_progress.connect(self.serverProgress)
-        
+
     def serverStatusChanged(self, server_status):
         self.close()
-        
+
     def serverProgress(self, value):
         self.ui.progressBar.setValue(value * 100)
-        
+
+
+class ScriptInfoDialog(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_script_info.Ui_Dialog()
+        self.ui.setupUi(self)
+
+    def setInfoLabel(self, text):
+        self.ui.infoLabel.setText(text)
+
+    def shouldBeRemoved(self):
+        return False
+
+
+class ScriptUserActionDialog(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_script_user_action.Ui_Dialog()
+        self.ui.setupUi(self)
+
+        self.ui.buttonBox.clicked.connect(self.buttonBoxClicked)
+        self.ui.infoLabel.setVisible(False)
+        self.ui.infoLine.setVisible(False)
+
+        self._is_terminated = False
+
+    def setMainText(self, text):
+        self.ui.label.setText(text)
+
+    def setInfoLabel(self, text):
+        self.ui.infoLabel.setText(text)
+        self.ui.infoLabel.setVisible(True)
+        self.ui.infoLine.setVisible(True)
+
+    def validate(self):
+        self.toDaemon('/reply', '/ray/gui/script_user_action',
+                      'Dialog window validated')
+        self._is_terminated = True
+        self.accept()
+
+    def abort(self):
+        self.toDaemon('/error', '/ray/gui/script_user_action',
+                      ray.Err.ABORT_ORDERED, 'Script user action aborted!')
+        self._is_terminated = True
+        self.accept()
+
+    def buttonBoxClicked(self, button):
+        if button == self.ui.buttonBox.button(QDialogButtonBox.Yes):
+            self.validate()
+        elif button == self.ui.buttonBox.button(QDialogButtonBox.Ignore):
+            self.abort()
+
+    def shouldBeRemoved(self):
+        return self._is_terminated
+
+
+class SessionScriptsInfoDialog(ChildDialog):
+    def __init__(self, parent, session_path):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_session_scripts_info.Ui_Dialog()
+        self.ui.setupUi(self)
+
+        scripts_dir = "%s/%s" % (session_path, ray.SCRIPTS_DIR)
+        parent_path = os.path.dirname(session_path)
+        parent_scripts = "%s/%s" % (parent_path, ray.SCRIPTS_DIR)
+
+        session_scripts_text = self.ui.textSessionScripts.toHtml()
+
+        self.ui.textSessionScripts.setHtml(
+            session_scripts_text % (scripts_dir, parent_scripts, parent_path))
+
+    def notAgainValue(self)->bool:
+        return self.ui.checkBoxNotAgain.isChecked()
+
+class JackConfigInfoDialog(ChildDialog):
+    def __init__(self, parent, session_path):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_jack_config_info.Ui_Dialog()
+        self.ui.setupUi(self)
+
+        scripts_dir = "%s/%s" % (session_path, ray.SCRIPTS_DIR)
+        parent_path = os.path.dirname(session_path)
+        parent_scripts = "%s/%s" % (parent_path, ray.SCRIPTS_DIR)
+
+        session_scripts_text = self.ui.textSessionScripts.toHtml()
+
+        self.ui.textSessionScripts.setHtml(
+            session_scripts_text % (scripts_dir, parent_scripts, parent_path))
+
+    def notAgainValue(self)->bool:
+        return self.ui.checkBoxNotAgain.isChecked()
+
+    def autostartValue(self)->bool:
+        return self.ui.checkBoxAutoStart.isChecked()
+
 
 class DaemonUrlWindow(ChildDialog):
     def __init__(self, parent, err_code, ex_url):
@@ -1112,30 +1135,30 @@ class WaitingCloseUserDialog(ChildDialog):
         ChildDialog.__init__(self, parent)
         self.ui = ui_waiting_close_user.Ui_Dialog()
         self.ui.setupUi(self)
-        
+
         if isDarkTheme(self):
             self.ui.labelSaveIcon.setPixmap(
                 QPixmap(':scalable/breeze-dark/document-nosave.svg'))
-        
+
         self.ui.pushButtonOk.setFocus(True)
         self.ui.pushButtonUndo.clicked.connect(self.undoClose)
         self.ui.pushButtonSkip.clicked.connect(self.skip)
         self.ui.checkBox.setChecked(
             bool(RS.settings.value(
                 'hide_wait_close_user_dialog', False, type=bool)))
-        
+
         self.ui.checkBox.clicked.connect(self.checkBoxClicked)
-        
+
     def serverStatusChanged(self, server_status):
         if server_status != ray.ServerStatus.WAIT_USER:
             self.accept()
-    
+
     def undoClose(self):
         self.toDaemon('/ray/session/cancel_close')
-    
+
     def skip(self):
         self.toDaemon('/ray/session/skip_wait_user')
-        
+
     def checkBoxClicked(self, state):
         RS.settings.setValue('hide_wait_close_user_dialog',
                              bool(state), type=bool)
@@ -1145,14 +1168,13 @@ class DonationsDialog(ChildDialog):
         ChildDialog.__init__(self, parent)
         self.ui = ui_donations.Ui_Dialog()
         self.ui.setupUi(self)
-        
-        print('display_no_again', display_no_again)
+
         self.ui.checkBox.setVisible(display_no_again)
         self.ui.checkBox.clicked.connect(self.checkBoxClicked)
-        
+
     def checkBoxClicked(self, state):
         RS.settings.setValue('hide_donations', state)
-        
+
 
 class ErrorDialog(ChildDialog):
     def __init__(self, parent, message):
@@ -1161,4 +1183,3 @@ class ErrorDialog(ChildDialog):
         self.ui.setupUi(self)
 
         self.ui.label.setText(message)
-
