@@ -1,4 +1,3 @@
-import os
 import socket
 import sys
 from liblo import Address
@@ -6,10 +5,8 @@ from PyQt5.QtCore import QObject, QProcess, QTimer
 from PyQt5.QtWidgets import QApplication
 
 import ray
-from gui_signaler import Signaler
 from gui_server_thread import GUIServerThread
-from gui_tools import (CommandLineArgs, default_session_root, ErrDaemon,
-                       _translate)
+from gui_tools import CommandLineArgs, ErrDaemon, _translate
 
 class DaemonManager(QObject):
     def __init__(self, session):
@@ -20,7 +17,6 @@ class DaemonManager(QObject):
         self.executable = 'ray-daemon'
         self.process = QProcess()
 
-        self.process.finished.connect(self.processFinished)
         if ray.QT_VERSION >= (5, 6):
             self.process.errorOccurred.connect(self.errorInProcess)
         self.process.setProcessChannelMode(QProcess.ForwardedChannels)
@@ -40,21 +36,14 @@ class DaemonManager(QObject):
         self.is_announced = False
         self.is_nsm_locked = False
 
+        self.session_root = ""
+
         self._signaler.daemon_announce.connect(self.receiveAnnounce)
         self._signaler.daemon_url_changed.connect(self.changeUrl)
 
     def finishInit(self):
         self._main_win = self._session._main_win
 
-    def processFinished(self, exit_code, exit_status):
-        if self.stopped_yet:
-            QApplication.quit()
-            return
-
-        if not self._main_win.isHidden():
-            self._main_win.daemonCrash()
-            return
-
     def errorInProcess(self, error):
         self._main_win.daemonCrash()
 
@@ -113,7 +102,8 @@ class DaemonManager(QObject):
             self.disannounce(src_addr)
             return
 
-        if CommandLineArgs.net_session_root and session_root != CommandLineArgs.net_session_root:
+        if (CommandLineArgs.net_session_root
+                and session_root != CommandLineArgs.net_session_root):
             self._signaler.daemon_url_request.emit(
                 ErrDaemon.WRONG_ROOT, self.url)
             self.disannounce(src_addr)
@@ -125,7 +115,8 @@ class DaemonManager(QObject):
             self.disannounce(src_addr)
             return
 
-        if CommandLineArgs.out_daemon and server_status != ray.ServerStatus.OFF:
+        if (CommandLineArgs.out_daemon
+                and server_status != ray.ServerStatus.OFF):
             self._signaler.daemon_url_request.emit(ErrDaemon.NOT_OFF, self.url)
             self.disannounce(src_addr)
             return
@@ -137,8 +128,6 @@ class DaemonManager(QObject):
         self.session_root = session_root
 
         self.is_nsm_locked = options & ray.Option.NSM_LOCKED
-        save_all_from_saved_client = options & ray.Option.SAVE_FROM_CLIENT
-        bookmark_session_folder = options & ray.Option.BOOKMARK_SESSION
 
         if self.is_nsm_locked:
             #self._signaler.daemon_nsm_locked.emit(True)
@@ -178,14 +167,28 @@ class DaemonManager(QObject):
     def setOscAddressViaUrl(self, url):
         self.setOscAddress(ray.getLibloAddress(url))
 
-    def processIsRunning(self):
-        return bool(self.process.state() == 2)
-
     def start(self):
         if self.launched_before:
             self.callDaemon()
             return
 
+        ray_control_process = QProcess()
+        ray_control_process.start("ray_control", ['get_port_gui_free'])
+        ray_control_process.waitForFinished(2000)
+
+        if ray_control_process.exitCode() == 0:
+            port_str_lines = ray_control_process.readAllStandardOutput().data().decode('utf-8')
+            port_str = port_str_lines.partition('\n')[0]
+
+            if port_str and port_str.isdigit():
+                self.address = Address(int(port_str))
+                self.port = self.address.port
+                self.url = self.address.url
+                self.launched_before = True
+                self.is_local = True
+                self.callDaemon()
+                return
+
         server = GUIServerThread.instance()
         if not server:
             sys.stderr.write(
@@ -195,7 +198,7 @@ class DaemonManager(QObject):
         arguments = ['--gui-url', str(server.url),
                      '--osc-port', str(self.port),
                      '--session-root', CommandLineArgs.session_root]
-        
+
         if CommandLineArgs.session:
             arguments.append('--session')
             arguments.append(CommandLineArgs.session)
@@ -204,25 +207,25 @@ class DaemonManager(QObject):
             arguments.append('--debug-only')
         elif CommandLineArgs.debug:
             arguments.append('--debug')
+        elif CommandLineArgs.no_client_messages:
+            arguments.append('--no-client-messages')
 
         if CommandLineArgs.config_dir:
             arguments.append('--config-dir')
             arguments.append(CommandLineArgs.config_dir)
 
-        self.process.start('ray-daemon', arguments)
+        self.process.startDetached('ray-daemon', arguments)
         #self.process.start('konsole', ['-e', 'ray-daemon'] + arguments)
 
     def stop(self):
         if self.launched_before:
             self.disannounce()
-            QApplication.quit()
+            QTimer.singleShot(10, QApplication.quit)
             return
 
-        if self.processIsRunning():
-            if not self.stopped_yet:
-                self.process.terminate()
-                self.stopped_yet = True
-                QTimer.singleShot(5000, self.notEndedAfterWait)
+        server = GUIServerThread.instance()
+        server.toDaemon('/ray/server/quit')
+        QTimer.singleShot(10, QApplication.quit)
 
     def notEndedAfterWait(self):
         sys.stderr.write('ray-daemon is still running, sorry !\n')
