@@ -2,35 +2,26 @@ import time
 import sys
 from PyQt5.QtCore import QObject, pyqtSignal
 
-import child_dialogs
-import snapshots_dialog
 import ray
 from gui_server_thread import GUIServerThread
+from client_properties_dialog import ClientPropertiesDialog
 
-
-class Client(QObject):
+class Client(QObject, ray.ClientData):
     status_changed = pyqtSignal(int)
-    
-    def __init__(self, session, client_data, trashed=False):
+
+    def __init__(self, session, client_id, protocol, trashed=False):
         QObject.__init__(self)
+        ray.ClientData.gui_init(self, client_id, protocol)
+
         self._session = session
         self._main_win = self._session._main_win
 
-        self.client_id       = client_data.client_id
-        self.executable_path = client_data.executable_path
-        self.arguments       = client_data.arguments
-        self.name            = client_data.name
-        self.prefix_mode     = client_data.prefix_mode
-        self.custom_prefix   = client_data.custom_prefix
-        self.label           = client_data.label
-        self.icon_name       = client_data.icon
-        self.capabilities    = client_data.capabilities
-        self.check_last_save = client_data.check_last_save
-        self.ignored_extensions = client_data.ignored_extensions
+        self.ray_hack = ray.RayHack()
 
         self.status = ray.ClientStatus.STOPPED
         self.previous_status = ray.ClientStatus.STOPPED
         self.hasGui = False
+        self.gui_state = False
         self.gui_visible = False
         self.has_dirty = False
         self.dirty_state = True
@@ -38,8 +29,8 @@ class Client(QObject):
         self.last_save = time.time()
 
         self.widget = self._main_win.createClientWidget(self)
-        self.properties_dialog = child_dialogs.ClientPropertiesDialog(
-            self._main_win, self)
+        self.properties_dialog = ClientPropertiesDialog.create(self._main_win,
+                                                               self)
 
     def setStatus(self, status):
         self.previous_status = self.status
@@ -53,31 +44,28 @@ class Client(QObject):
             self.last_save = time.time()
 
         self.widget.updateStatus(status)
+        self.properties_dialog.updateStatus(status)
 
     def setGuiEnabled(self):
         self.hasGui = True
         self.widget.showGuiButton()
 
-    def setGuiState(self, state):
+    def setGuiState(self, state: bool):
         self.gui_state = state
         self.widget.setGuiState(state)
 
-    def setDirtyState(self, bool_dirty):
+    def setDirtyState(self, dirty: bool):
         self.has_dirty = True
-        self.dirty_state = bool_dirty
-        self.widget.setDirtyState(bool_dirty)
+        self.dirty_state = dirty
+        self.widget.setDirtyState(dirty)
 
     def setNoSaveLevel(self, no_save_level):
         self.no_save_level = no_save_level
         self.widget.setNoSaveLevel(no_save_level)
-    
+
     def setProgress(self, progress):
         self.widget.setProgress(progress)
 
-    def switch(self, new_client_id):
-        self.client_id = new_client_id
-        self.widget.updateClientData()
-
     def allowKill(self):
         self.widget.allowKill()
 
@@ -85,17 +73,12 @@ class Client(QObject):
         self.label = label
         self.sendPropertiesToDaemon()
 
-    def updateClientProperties(self, client_data):
-        self.executable_path = client_data.executable_path
-        self.arguments       = client_data.arguments
-        self.name            = client_data.name
-        self.prefix_mode     = client_data.prefix_mode
-        self.custom_prefix   = client_data.custom_prefix
-        self.label           = client_data.label
-        self.icon_name       = client_data.icon
-        self.capabilities    = client_data.capabilities
-        self.check_last_save = client_data.check_last_save
-        
+    def updateClientProperties(self, *args):
+        self.update(*args)
+        self.widget.updateClientData()
+
+    def updateRayHack(self, *args):
+        self.ray_hack.update(*args)
         self.widget.updateClientData()
 
     def prettierName(self):
@@ -116,20 +99,26 @@ class Client(QObject):
             return
 
         server.toDaemon('/ray/client/update_properties',
-                        self.client_id,
-                        self.executable_path,
-                        self.arguments,
-                        self.name,
-                        self.prefix_mode,
-                        self.custom_prefix,
-                        self.label,
-                        self.icon_name,
-                        self.capabilities,
-                        int(self.check_last_save),
-                        self.ignored_extensions)
+                        *ray.ClientData.spreadClient(self))
 
-    def showPropertiesDialog(self):
+    def sendRayHack(self):
+        if self.protocol != ray.Protocol.RAY_HACK:
+            return
+
+        server = GUIServerThread.instance()
+        if not server:
+            return
+
+        server.toDaemon('/ray/client/update_ray_hack_properties',
+                        self.client_id,
+                        *self.ray_hack.spread())
+
+    def showPropertiesDialog(self, second_tab=False):
         self.properties_dialog.updateContents()
+        if second_tab:
+            if self.protocol == ray.Protocol.RAY_HACK:
+                self.properties_dialog.enableTestZone(True)
+            self.properties_dialog.setOnSecondTab()
         self.properties_dialog.show()
         self.properties_dialog.activateWindow()
 
@@ -142,14 +131,44 @@ class Client(QObject):
             self.setGuiEnabled()
 
     def hasBeenRecentlySaved(self):
-        if (time.time() - self.last_save) >= 60:  
+        if (time.time() - self.last_save) >= 60:
             # last save more than 60 seconds ago
             return False
 
         return True
 
+    def getProjectPath(self)->str:
+        if not self._session.path:
+            return ''
 
-class TrashedClient(object):
+        prefix = self.session.name
+
+        if self.prefix_mode == ray.PrefixMode.CLIENT_NAME:
+            prefix = self.name
+        elif self.prefix_mode == ray.PrefixMode.CUSTOM:
+            prefix = self.custom_prefix
+
+        return "%s/%s.%s" % (self._session.path, prefix, self.client_id)
+
+    def getIconSearchPath(self)->list:
+        if not self._session._daemon_manager.is_local:
+            return []
+
+        project_path = self.getProjectPath()
+        if not project_path:
+            return []
+
+        search_list = []
+        main_icon_path = '.local/share/icons'
+        search_list.append("%s/%s" % (search_list, main_icon_path))
+
+        for path in ('16x16', '24x24', '32x32', '64x64', 'scalable'):
+            search_list.append("%s/%s/%s" % (project_path,
+                                             main_icon_path, path))
+        return search_list
+
+
+class TrashedClient:
     def __init__(self, client_data, menu_action):
         self.data = client_data
         self.menu_action = menu_action
