@@ -21,8 +21,15 @@ class Signaler(QObject):
 
     daemon_url_request = pyqtSignal(int, str)
     daemon_url_changed = pyqtSignal(str)
-    
+
+    client_template_update = pyqtSignal(list)
+
     root_changed = pyqtSignal(str)
 
+    client_added_reply = pyqtSignal(str)
+
+    client_properties_state_changed = pyqtSignal(str, bool)
+
+
     def __init__(self):
         QObject.__init__(self)
