@@ -18,13 +18,13 @@ def _translate(*args):
 def setDefaultSessionRoot(path):
     global default_session_root
     default_session_root = path
-    
+
 def getDefaultSessionRoot():
     return default_session_root
 
 class RS:
     settings = QSettings()
-    
+
     @classmethod
     def setSettings(cls, settings):
         del cls.settings
@@ -38,9 +38,9 @@ def initGuiTools():
         settings = QSettings(CommandLineArgs.config_dir)
     else:
         settings = QSettings()
-       
+
     RS.setSettings(settings)
-    
+
     CommandLineArgs.changeSessionRoot(
         settings.value('default_session_root',
                        "%s/Ray Sessions" % (os.getenv('HOME')),
@@ -48,7 +48,7 @@ def initGuiTools():
 
 def isDarkTheme(widget):
     return bool(
-        widget.palette().brush(2, QPalette.WindowText).color().lightness()
+        widget.palette().brush(QPalette.Active, QPalette.WindowText).color().lightness()
         > 128)
 
 def dirname(*args):
@@ -73,8 +73,9 @@ def serverStatusString(server_status):
         ray.ServerStatus.SNAPSHOT: _translate('server_status', "snapshot"),
         ray.ServerStatus.REWIND  : _translate('server_status', "rewind"),
         ray.ServerStatus.WAIT_USER : _translate('server_status', "waiting"),
-        ray.ServerStatus.OUT_SAVE: _translate('server_status', "save"),
-        ray.ServerStatus.OUT_SNAPSHOT: _translate('server_status', "snapshot")}
+        ray.ServerStatus.OUT_SAVE  : _translate('server_status', "save"),
+        ray.ServerStatus.OUT_SNAPSHOT: _translate('server_status', "snapshot"),
+        ray.ServerStatus.SCRIPT : _translate('server_status', "script")}
 
     if not 0 <= server_status < len(server_status_strings):
         return _translate('server status', "invalid")
@@ -96,7 +97,8 @@ def clientStatusString(client_status):
         ray.ClientStatus.NOOP   : _translate('client status', "noop"),
         ray.ClientStatus.ERROR  : _translate('client status', "error"),
         ray.ClientStatus.REMOVED: _translate('client status', "removed"),
-        ray.ClientStatus.UNDEF  : _translate('client_status', "")}
+        ray.ClientStatus.UNDEF  : _translate('client_status', ""),
+        ray.ClientStatus.SCRIPT : _translate('client_status', 'script')}
 
     if not 0 <= client_status < len(client_status_strings):
         return _translate('client_status', 'invalid')
@@ -124,7 +126,7 @@ class RayIcon(QIcon):
             QPixmap(
                 ':scalable/%s/disabled/%s' %
                 (breeze, icon_name)), QIcon.Disabled, QIcon.Off)
-    
+
 
 class CommandLineArgs(argparse.Namespace):
     daemon_url = None
@@ -132,6 +134,7 @@ class CommandLineArgs(argparse.Namespace):
     config_dir = ''
     debug = False
     debug_only = False
+    no_client_messages = False
     net_session_root = ''
     net_daemon_id = 0
     under_nsm = False
@@ -163,15 +166,15 @@ class CommandLineArgs(argparse.Namespace):
                 sys.exit(1)
 
             cls.under_nsm = True
-        
+
         if not cls.session_root:
             cls.session_root = RS.settings.value(
                                     'default_session_root',
                                     '%s/Ray Sessions' % os.getenv('HOME'))
-        
+
         if cls.session_root.endswith('/'):
             cls.session_root = cls.session_root[:-1]
-            
+
     @classmethod
     def changeSessionRoot(cls, path):
         cls.session_root = path
@@ -188,13 +191,17 @@ class ArgParser(argparse.ArgumentParser):
         self.add_argument('--daemon-url', '-u', type=ray.getLibloAddress,
                           help=_translate('help',
                                           'connect to this daemon url'))
+        self.add_argument('--daemon-port', '-p',
+                          type=ray.getLibloAddressFromPort,
+                          help=_translate('help',
+                                          'connect to this daemon port'))
         self.add_argument('--out-daemon', action='store_true',
                           help=argparse.SUPPRESS)
         self.add_argument('--session-root', '-r', type=str,
                           help=_translate(
                               'help', 'Use this folder as root for sessions'))
         self.add_argument('--session', '-s', type=str,
-                          help=_translate('help', 
+                          help=_translate('help',
                                           'Open this session at startup'))
         self.add_argument('--config-dir', '-c', type=str, default='',
                           help=_translate('help', 'use a custom config dir'))
@@ -203,6 +210,9 @@ class ArgParser(argparse.ArgumentParser):
         self.add_argument('--debug-only', '-do', action='store_true',
                           help=_translate('help',
                                           'debug without client messages'))
+        self.add_argument('---no-client-messages', '-ncm', action='store_true',
+                          help=_translate('help',
+                                          'do not print client messages'))
         self.add_argument('--net-session-root', type=str, default='',
                           help=argparse.SUPPRESS)
         self.add_argument('--net-daemon-id', type=int, default=0,
