@@ -1,8 +1,6 @@
-import time
-from PyQt5.QtWidgets import (QListWidget, QListWidgetItem, QFrame, QMenu, 
-                             QLineEdit, QAction)
-from PyQt5.QtGui import QIcon, QPalette, QPixmap, QFontMetrics, QFont, QFontDatabase
-from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot, QSize, QFile
+from PyQt5.QtWidgets import QListWidget, QListWidgetItem, QFrame, QMenu
+from PyQt5.QtGui import QIcon, QPixmap, QFont, QFontDatabase
+from PyQt5.QtCore import pyqtSlot, QSize
 
 import ray
 from gui_server_thread import GUIServerThread
@@ -15,6 +13,12 @@ import ui_client_slot
 
 
 class ClientSlot(QFrame):
+    @classmethod
+    def toDaemon(cls, *args):
+        server = GUIServerThread.instance()
+        if server:
+            server.toDaemon(*args)
+    
     def __init__(self, list_widget, client):
         QFrame.__init__(self)
         self.ui = ui_client_slot.Ui_ClientSlotWidget()
@@ -26,9 +30,13 @@ class ClientSlot(QFrame):
         self._main_win = self.client._session._main_win
 
         self.is_dirty_able = False
-        self.gui_visible = True
+        self.gui_visible = False
+
         self.ui.toolButtonGUI.setVisible(False)
 
+        if self.client.protocol == ray.Protocol.RAY_HACK:
+            self.showGuiButton()
+
         # connect buttons to functions
         self.ui.toolButtonGUI.toggleGui.connect(self.toggleGui)
         self.ui.startButton.clicked.connect(self.startClient)
@@ -38,7 +46,7 @@ class ClientSlot(QFrame):
         self.ui.closeButton.clicked.connect(self.trashClient)
         self.ui.lineEditClientStatus.statusPressed.connect(self.abortCopy)
         # self.ui.ClientName.name_changed.connect(self.updateLabel)
-        
+
         # prevent "stopped" status displayed at client switch
         #self.ui.lineEditClientStatus,.setTextNow(
                           #clientStatusString(self.client.status))
@@ -46,8 +54,6 @@ class ClientSlot(QFrame):
         self.icon_on = QIcon()
         self.icon_off = QIcon()
 
-        self.updateClientData()
-
         self.ui.actionSaveAsApplicationTemplate.triggered.connect(
             self.saveAsApplicationTemplate)
         self.ui.actionProperties.triggered.connect(
@@ -60,7 +66,7 @@ class ClientSlot(QFrame):
         self.menu.addAction(self.ui.actionSaveAsApplicationTemplate)
         self.menu.addAction(self.ui.actionReturnToAPreviousState)
         self.menu.addAction(self.ui.actionProperties)
-        
+
         self.ui.actionReturnToAPreviousState.setVisible(
             self._main_win.has_git)
 
@@ -88,7 +94,7 @@ class ClientSlot(QFrame):
             QPixmap(':scalable/breeze/document-unsaved'),
             QIcon.Normal,
             QIcon.Off)
-        
+
         self.noSaveIcon = QIcon()
         self.noSaveIcon.addPixmap(
             QPixmap(':scalable/breeze/document-nosave'),
@@ -141,7 +147,7 @@ class ClientSlot(QFrame):
                 QPixmap(':scalable/breeze-dark/document-unsaved'),
                 QIcon.Normal,
                 QIcon.Off)
-            
+
             self.noSaveIcon = QIcon()
             self.noSaveIcon.addPixmap(
                 QPixmap(':scalable/breeze-dark/document-nosave'),
@@ -168,13 +174,11 @@ class ClientSlot(QFrame):
 
         self.ui.killButton.setVisible(False)
 
+        self.updateClientData()
+
     def clientId(self):
         return self.client.client_id
 
-    def toDaemon(self, *args):
-        server = GUIServerThread.instance()
-        if server:
-            server.toDaemon(*args)
 
     def startClient(self):
         self.toDaemon('/ray/client/resume', self.clientId())
@@ -205,7 +209,7 @@ class ClientSlot(QFrame):
         template_name = dialog.getTemplateName()
         self.toDaemon('/ray/client/save_as_template', self.clientId(),
                       template_name)
-        
+
     def openSnapshotsDialog(self):
         dialog = snapshots_dialog.ClientSnapshotsDialog(self._main_win,
                                                         self.client)
@@ -224,13 +228,26 @@ class ClientSlot(QFrame):
         self.ui.ClientName.setText(label)
 
         # set tool tip
-        self.ui.ClientName.setToolTip(
-            'Executable : '
-                + self.client.executable_path + '\n'
-                + 'Client id : ' + self.clientId())
+        tool_tip = "<html><head/><body>"
+        tool_tip += "<p><span style=\" font-weight:600;\">%s<br></span>" \
+            % self.client.name
+        tool_tip += "<span style=\" font-style:italic;\">%s</span></p>" \
+            % self.client.description
+        tool_tip += "<p></p>"
+        tool_tip += "<p>%s : %s<br>" \
+            % (_translate('client_slot', 'Protocol'),
+               ray.protocolToStr(self.client.protocol))
+        tool_tip += "%s : %s<br>" \
+            % (_translate('client_slot', 'Executable'),
+               self.client.executable_path)
+        tool_tip += "%s : %s</p>" \
+            % (_translate('client_slot', 'client id'), self.client.client_id)
+        tool_tip += "</body></html>"
+
+        self.ui.ClientName.setToolTip(tool_tip)
 
         # set icon
-        self.icon_on = ray.getAppIcon(self.client.icon_name, self)
+        self.icon_on = ray.getAppIcon(self.client.icon, self)
         self.icon_off = QIcon(self.icon_on.pixmap(32, 32, QIcon.Disabled))
 
         self.grayIcon(
@@ -247,7 +264,9 @@ class ClientSlot(QFrame):
 
     def updateStatus(self, status):
         self.ui.lineEditClientStatus.setText(clientStatusString(status))
-        
+
+        ray_hack = bool(self.client.protocol == ray.Protocol.RAY_HACK)
+
         if status in (
                 ray.ClientStatus.LAUNCH,
                 ray.ClientStatus.OPEN,
@@ -279,7 +298,7 @@ class ClientSlot(QFrame):
             self.ui.closeButton.setEnabled(True)
             self.ui.ClientName.setStyleSheet('QLabel {font-weight : normal}')
             self.ui.ClientName.setEnabled(False)
-            self.ui.toolButtonGUI.setEnabled(False)
+            self.ui.toolButtonGUI.setEnabled(ray_hack)
             self.grayIcon(True)
 
             self.ui.stopButton.setVisible(True)
@@ -294,7 +313,7 @@ class ClientSlot(QFrame):
             self.ui.closeButton.setEnabled(True)
             self.ui.ClientName.setStyleSheet('QLabel {font-weight : normal}')
             self.ui.ClientName.setEnabled(False)
-            self.ui.toolButtonGUI.setEnabled(False)
+            self.ui.toolButtonGUI.setEnabled(ray_hack)
             self.grayIcon(True)
 
             self.ui.stopButton.setVisible(True)
@@ -318,7 +337,14 @@ class ClientSlot(QFrame):
 
     def showGuiButton(self):
         self.ui.toolButtonGUI.setVisible(True)
-        if self.client.executable_path in ('nsm-proxy', 'ray-proxy'):
+        if self.client.protocol == ray.Protocol.RAY_HACK:
+            self.ui.toolButtonGUI.setText(
+                _translate('client_slot', 'Hack'))
+            self.ui.toolButtonGUI.setToolTip(
+                _translate('client_slot',
+                           'Display client Ray-Hack properties'))
+
+        elif self.client.executable_path in ('nsm-proxy', 'ray-proxy'):
             self.ui.toolButtonGUI.setText(_translate('client_slot', 'proxy'))
             self.ui.toolButtonGUI.setToolTip(
                 _translate('client_slot', 'Display proxy window'))
@@ -328,10 +354,19 @@ class ClientSlot(QFrame):
         self.ui.toolButtonGUI.setChecked(state)
 
     def toggleGui(self):
-        if not self.gui_visible:
-            self.toDaemon('/ray/client/show_optional_gui', self.clientId())
+        if self.client.protocol == ray.Protocol.RAY_HACK:
+            if self.gui_visible:
+                self.client.properties_dialog.hide()
+            else:
+                self.client.showPropertiesDialog(second_tab=True)
+
         else:
-            self.toDaemon('/ray/client/hide_optional_gui', self.clientId())
+            if not self.gui_visible:
+                self.toDaemon('/ray/client/show_optional_gui',
+                              self.clientId())
+            else:
+                self.toDaemon('/ray/client/hide_optional_gui',
+                              self.clientId())
 
     def setDirtyState(self, bool_dirty):
         self.is_dirty_able = True
@@ -346,10 +381,10 @@ class ClientSlot(QFrame):
             self.ui.saveButton.setIcon(self.noSaveIcon)
         else:
             self.ui.saveButton.setIcon(self.saveIcon)
-    
+
     def setProgress(self, progress):
         self.ui.lineEditClientStatus.setProgress(progress)
-        
+
     def setDaemonOptions(self, options):
         has_git = bool(options & ray.Option.HAS_GIT)
         self.ui.actionReturnToAPreviousState.setVisible(has_git)
@@ -382,11 +417,17 @@ class ClientItem(QListWidgetItem):
 
 
 class ListWidgetClients(QListWidget):
+    @classmethod
+    def toDaemon(self, *args):
+        server = GUIServerThread.instance()
+        if server:
+            server.toDaemon(*args)
+    
     def __init__(self, parent):
         QListWidget.__init__(self, parent)
         self.last_n = 0
         self._session = None
-        
+
     def createClientWidget(self, client_data):
         item = ClientItem(self, client_data)
         item.setSortNumber(self.last_n)
@@ -414,8 +455,6 @@ class ListWidgetClients(QListWidget):
             else:
                 return
 
-        next_item_list = []
-
         n = 0
 
         for client_id in client_id_list:
@@ -426,15 +465,18 @@ class ListWidgetClients(QListWidget):
             n += 1
 
         self.sortItems()
-    
+
+    def clientPropertiesStateChanged(self, client_id, bool_visible):
+        for i in range(self.count()):
+            item = self.item(i)
+            if item.getClientId() == client_id:
+                widget = item.f_widget
+                widget.setGuiState(bool_visible)
+                break
+
     def setSession(self, session):
         self._session = session
-    
-    def toDaemon(self, *args):
-        server = GUIServerThread.instance()
-        if server:
-            server.toDaemon(*args)
-    
+
     @pyqtSlot()
     def launchFavorite(self):
         template_name, factory = self.sender().data()
@@ -460,13 +502,13 @@ class ListWidgetClients(QListWidget):
     def mousePressEvent(self, event):
         if not self.itemAt(event.pos()):
             self.setCurrentRow(-1)
-            
+
         QListWidget.mousePressEvent(self, event)
-    
+
     def contextMenuEvent(self, event):
         if not self.itemAt(event.pos()):
             self.setCurrentRow(-1)
-            
+
             if (self._session
                     and not self._session.server_status in (
                         ray.ServerStatus.OFF,
@@ -477,21 +519,19 @@ class ListWidgetClients(QListWidget):
                 menu = QMenu()
                 fav_menu = QMenu(_translate('menu', 'Favorites'), menu)
                 fav_menu.setIcon(QIcon(':scalable/breeze/star-yellow'))
-                
+
                 for favorite in self._session.favorite_list:
                     act_app = fav_menu.addAction(
                         ray.getAppIcon(favorite.icon, self), favorite.name)
                     act_app.setData([favorite.name, favorite.factory])
                     act_app.triggered.connect(self.launchFavorite)
-            
+
                 menu.addMenu(fav_menu)
-                
+
                 menu.addAction(
                     self._session._main_win.ui.actionAddApplication)
                 menu.addAction(self._session._main_win.ui.actionAddExecutable)
-            
+
                 act_selected = menu.exec(self.mapToGlobal(event.pos()))
             event.accept()
             return
-        
-        
