@@ -1,11 +1,8 @@
 import time
-from PyQt5.QtWidgets import (QApplication, QMainWindow, QMenu, QInputDialog,
-        QBoxLayout, QListWidgetItem, QFrame, QDialog, QDialogButtonBox,
-        QFileDialog, QMessageBox, QCompleter, QAction, QToolButton,
-        QAbstractItemView, QLabel, QLineEdit)
-from PyQt5.QtGui import QIcon, QCursor, QPalette, QPixmap, QFontDatabase
-from PyQt5.QtCore import (QTimer, QProcess, pyqtSignal, pyqtSlot, QObject,
-                          QSize, Qt, QSettings, qDebug, QLocale, QTranslator)
+from PyQt5.QtWidgets import (QApplication, QMainWindow, QMenu, QDialog,
+                             QMessageBox, QToolButton, QAbstractItemView)
+from PyQt5.QtGui import QIcon
+from PyQt5.QtCore import QTimer, pyqtSlot
 
 from gui_tools import (RS, RayIcon, CommandLineArgs, _translate,
                        serverStatusString, isDarkTheme)
@@ -13,16 +10,20 @@ import add_application_dialog
 import child_dialogs
 import snapshots_dialog
 from gui_server_thread import GUIServerThread
-from gui_client import TrashedClient
 
 import ray
 import list_widget_clients
 
 import ui_raysession
-import ui_client_slot
 
 
 class MainWindow(QMainWindow):
+    @classmethod
+    def toDaemon(cls, *args):
+        server = GUIServerThread.instance()
+        if server:
+            server.toDaemon(*args)
+    
     def __init__(self, session):
         QMainWindow.__init__(self)
         self.ui = ui_raysession.Ui_MainWindow()
@@ -46,20 +47,37 @@ class MainWindow(QMainWindow):
         self.timer_flicker_open.timeout.connect(self.flashOpen)
         self.flash_open_list = []
         self.flash_open_bool = False
-        
+
         # timer for too long snapshots, display snapshot progress dialog
         self.timer_snapshot = QTimer()
         self.timer_snapshot.setSingleShot(True)
         self.timer_snapshot.setInterval(2000)
         self.timer_snapshot.timeout.connect(self.showSnapshotProgressDialog)
-        
+
 
         self.server_copying = False
 
         self.keep_focus = RS.settings.value('keepfocus', True, type=bool)
         self.ui.actionKeepFocus.setChecked(self.keep_focus)
+
+        if ray.getWindowManager() == ray.WindowManager.WAYLAND:
+            # do not enable keep focus option under Wayland
+            # because activate a window from it self on Wayland not allowed
+            self.keep_focus = False
+            self.ui.actionKeepFocus.setEnabled(False)
+
         if RS.settings.value('MainWindow/geometry'):
             self.restoreGeometry(RS.settings.value('MainWindow/geometry'))
+        else:
+            # first start (or start without config)
+            # set window as little as possible
+            rect = self.geometry()
+            x = rect.x()
+            y = rect.y()
+            height = rect.height()
+            self.setMinimumWidth(450)
+            self.setGeometry(x, y, 460, height)
+
         if RS.settings.value('MainWindow/WindowState'):
             self.restoreState(RS.settings.value('MainWindow/WindowState'))
         self.ui.actionShowMenuBar.activate(RS.settings.value(
@@ -84,6 +102,9 @@ class MainWindow(QMainWindow):
         self.ui.toolButtonSnapshots.setDefaultAction(
             self.ui.actionReturnToAPreviousState)
 
+        self.ui.dockWidgetMessages.visibilityChanged.connect(
+            self.resizeWinWithMessages)
+
         # connect actions
         self.ui.actionNewSession.triggered.connect(self.createNewSession)
         self.ui.actionOpenSession.triggered.connect(self.openSession)
@@ -104,14 +125,14 @@ class MainWindow(QMainWindow):
         self.ui.actionAddApplication.triggered.connect(self.addApplication)
         self.ui.actionAddExecutable.triggered.connect(self.addExecutable)
         self.ui.actionKeepFocus.toggled.connect(self.toggleKeepFocus)
-        self.ui.actionSaveAllFromSavedClient.triggered.connect(
-            self.saveAllFromClientToggled)
         self.ui.actionBookmarkSessionFolder.triggered.connect(
             self.bookmarkSessionFolderToggled)
         self.ui.actionDesktopsMemory.triggered.connect(
             self.desktopsMemoryToggled)
         self.ui.actionAutoSnapshot.triggered.connect(
             self.autoSnapshotToggled)
+        self.ui.actionSessionScripts.triggered.connect(
+            self.sessionScriptsToggled)
         self.ui.actionAboutRaySession.triggered.connect(self.aboutRaySession)
         self.ui.actionAboutQt.triggered.connect(QApplication.aboutQt)
         self.ui.actionDonate.triggered.connect(self.donate)
@@ -119,8 +140,8 @@ class MainWindow(QMainWindow):
         self.ui.lineEditServerStatus.statusPressed.connect(
             self.statusBarPressed)
         self.ui.stackedWidgetSessionName.name_changed.connect(
-            self.renameSession)
-        
+            self.renameSessionConditionnaly)
+
         # set control menu
         self.controlMenu = QMenu()
         self.controlMenu.addAction(self.ui.actionShowMenuBar)
@@ -128,10 +149,10 @@ class MainWindow(QMainWindow):
         self.controlMenu.addSeparator()
         self.controlMenu.addAction(self.ui.actionKeepFocus)
         self.controlMenu.addSeparator()
-        self.controlMenu.addAction(self.ui.actionSaveAllFromSavedClient)
         self.controlMenu.addAction(self.ui.actionBookmarkSessionFolder)
         self.controlMenu.addAction(self.ui.actionAutoSnapshot)
         self.controlMenu.addAction(self.ui.actionDesktopsMemory)
+        self.controlMenu.addAction(self.ui.actionSessionScripts)
 
         self.controlToolButton = self.ui.toolBar.widgetForAction(
             self.ui.actionControlMenu)
@@ -140,14 +161,14 @@ class MainWindow(QMainWindow):
 
         self.ui.toolButtonControl2.setPopupMode(QToolButton.InstantPopup)
         self.ui.toolButtonControl2.setMenu(self.controlMenu)
-        
+
         # set favorites menu
         self.favorites_menu = QMenu(_translate('menu', 'Favorites'))
         self.favorites_menu.setIcon(QIcon(':scalable/breeze/star-yellow'))
         self.ui.toolButtonFavorites.setPopupMode(QToolButton.InstantPopup)
         self.ui.toolButtonFavorites.setMenu(self.favorites_menu)
         self.ui.menuAdd.addMenu(self.favorites_menu)
-        
+
         # set trash menu
         self.trashMenu = QMenu()
         self.ui.trashButton.setPopupMode(QToolButton.InstantPopup)
@@ -159,36 +180,38 @@ class MainWindow(QMainWindow):
         sg.server_status_changed.connect(self.serverChangeServerStatus)
         sg.server_copying.connect(self.serverCopying)
         sg.daemon_url_request.connect(self.showDaemonUrlWindow)
-        
+        sg.client_properties_state_changed.connect(
+            self.clientPropertiesStateChanged)
+
         # set spare icons if system icons not avalaible
         dark = isDarkTheme(self)
-        
+
         if self.ui.actionNewSession.icon().isNull():
             self.ui.actionNewSession.setIcon(RayIcon('folder-new', dark))
         if self.ui.actionOpenSession.icon().isNull():
             self.ui.actionOpenSession.setIcon(RayIcon('document-open', dark))
-            
+
         if self.ui.actionControlMenu.icon().isNull():
             self.ui.actionControlMenu.setIcon(
                 QIcon.fromTheme('configuration_section'))
             if self.ui.actionControlMenu.icon().isNull():
                 self.ui.actionControlMenu.setIcon(RayIcon('configure', dark))
-        
+
         if self.ui.actionOpenSessionFolder.icon().isNull():
             self.ui.actionOpenSessionFolder.setIcon(
                                         RayIcon('system-file-manager', dark))
-        
+
         if self.ui.actionAddApplication.icon().isNull():
             self.ui.actionAddApplication.setIcon(RayIcon('list-add', dark))
-        
+
         if self.ui.actionAddExecutable.icon().isNull():
             self.ui.actionAddExecutable.setIcon(QIcon.fromTheme('system-run'))
             if self.ui.actionAddExecutable.icon().isNull():
                 self.ui.actionAddExecutable.setIcon(RayIcon('run-install'))
-            
+
         self.ui.actionReturnToAPreviousState.setIcon(
                                         RayIcon('media-seek-backward', dark))
-        
+
         self.ui.trashButton.setIcon(RayIcon('trash-empty', dark))
 
         self.ui.actionDuplicateSession.setIcon(
@@ -203,24 +226,27 @@ class MainWindow(QMainWindow):
 
         self.setNsmLocked(CommandLineArgs.under_nsm)
 
+        self.script_info_dialog = None
+        self.script_action_dialog = None
+
         # disable "keep focus" if daemon is not on this machine (it takes no
         # sense in this case)
         if not self._daemon_manager.is_local:
             self.ui.actionKeepFocus.setChecked(False)
             self.ui.actionKeepFocus.setEnabled(False)
-        
+
         self.server_progress = 0.0
         self.progress_dialog_visible = False
-        
+
         self.has_git = False
 
     def createClientWidget(self, client):
         return self.ui.listWidget.createClientWidget(client)
 
     def reCreateListWidget(self):
-        # this function shouldn't exist, 
+        # this function shouldn't exist,
         # it is a workaround for a bug with python-qt.
-        # (when reorder widgets sometimes one widget is totally hidden 
+        # (when reorder widgets sometimes one widget is totally hidden
         # until user resize the window)
         # It has to be modified when ui_raysession is modified.
 
@@ -259,21 +285,21 @@ class MainWindow(QMainWindow):
             nsm_locked and not CommandLineArgs.out_daemon)
 
     def setDaemonOptions(self, options):
-        self.ui.actionSaveAllFromSavedClient.setChecked(
-            bool(options & ray.Option.SAVE_FROM_CLIENT))
         self.ui.actionBookmarkSessionFolder.setChecked(
             bool(options & ray.Option.BOOKMARK_SESSION))
         self.ui.actionDesktopsMemory.setChecked(
             bool(options & ray.Option.DESKTOPS_MEMORY))
         self.ui.actionAutoSnapshot.setChecked(
             bool(options & ray.Option.SNAPSHOTS))
+        self.ui.actionSessionScripts.setChecked(
+            bool(options & ray.Option.SESSION_SCRIPTS))
 
         has_wmctrl = bool(options & ray.Option.HAS_WMCTRL)
         self.ui.actionDesktopsMemory.setEnabled(has_wmctrl)
         if has_wmctrl:
             self.ui.actionDesktopsMemory.setText(
                 _translate('actions', 'Desktops Memory'))
-            
+
         has_git = bool(options & ray.Option.HAS_GIT)
         self.ui.actionAutoSnapshot.setEnabled(has_git)
         self.ui.actionReturnToAPreviousState.setVisible(has_git)
@@ -281,13 +307,8 @@ class MainWindow(QMainWindow):
         if has_git:
             self.ui.actionAutoSnapshot.setText(
                 _translate('actions', 'Auto Snapshot at Save'))
-        
-        self.has_git = has_git
 
-    def toDaemon(self, *args):
-        server = GUIServerThread.instance()
-        if server:
-            server.toDaemon(*args)
+        self.has_git = has_git
 
     def hideMessagesDock(self):
         self.ui.dockWidgetMessages.setVisible(False)
@@ -299,16 +320,13 @@ class MainWindow(QMainWindow):
         if self.mouse_is_inside:
             self.activateWindow()
 
-    def toggleKeepFocus(self, bool):
-        self.keep_focus = bool
+    def toggleKeepFocus(self, keep_focus: bool):
+        self.keep_focus = keep_focus
         if self._daemon_manager.is_local:
             RS.settings.setValue('keepfocus', self.keep_focus)
-        if not bool:
+        if not keep_focus:
             self.timer_raisewin.stop()
 
-    def saveAllFromClientToggled(self, state):
-        self.toDaemon('/ray/option/save_from_client', int(state))
-
     def bookmarkSessionFolderToggled(self, state):
         self.toDaemon('/ray/option/bookmark_session_folder', int(state))
 
@@ -317,7 +335,10 @@ class MainWindow(QMainWindow):
 
     def autoSnapshotToggled(self, state):
         self.toDaemon('/ray/option/snapshots', int(state))
-        
+
+    def sessionScriptsToggled(self, state):
+        self.toDaemon('/ray/option/session_scripts', int(state))
+
     def flashOpen(self):
         for client in self._session.client_list:
             if client.status == ray.ClientStatus.OPEN:
@@ -326,17 +347,17 @@ class MainWindow(QMainWindow):
         self.flash_open_bool = not self.flash_open_bool
 
     def quitApp(self):
-        if (self._daemon_manager.launched_before
-                and not CommandLineArgs.under_nsm):
-            self.quitAppNow()
-            return True
+        #if (self._daemon_manager.launched_before
+                #and not CommandLineArgs.under_nsm):
+            #self.quitAppNow()
+            #return True
 
         if self._session.isRunning():
             dialog = child_dialogs.QuitAppDialog(self)
             dialog.exec()
             if not dialog.result():
                 return False
-            
+
         self.quitAppNow()
         return True
 
@@ -357,20 +378,67 @@ class MainWindow(QMainWindow):
         if not dialog.result():
             return
 
-        session_name = dialog.getSessionName()
+        session_short_path = dialog.getSessionShortPath()
         template_name = dialog.getTemplateName()
-        subfolder = dialog.getSubFolder()
+        subfolder = session_short_path.rpartition('/')[0]
 
         RS.settings.setValue('last_used_template', template_name)
         RS.settings.setValue('last_subfolder', subfolder)
         if self._session.isRunning():
-            RS.settings.setValue('last_session', self._session.getShortPath())
+            # remember the running session as last session (if any)
+            short_path = self._session.getShortPath()
+            if not short_path.startswith('/'):
+                RS.settings.setValue('last_session', short_path)
 
-        if template_name:
-            self.toDaemon('/ray/server/new_session', session_name,
-                          template_name)
-        else:
-            self.toDaemon('/ray/server/new_session', session_name)
+        if not template_name:
+            self.toDaemon('/ray/server/new_session', session_short_path)
+            return
+
+        if template_name.startswith('///'):
+            if template_name == '///' + ray.factory_session_templates[1]:
+                if not RS.settings.value('hide_jack_config_script_dialog',
+                                        False, type=bool):
+                    # display jack_config_script info dialog
+                    # and manage ray-jack_checker auto_start
+
+                    session_path = "%s/%s" % (CommandLineArgs.session_root,
+                                              session_short_path)
+
+                    dialog = child_dialogs.JackConfigInfoDialog(
+                                                        self, session_path)
+                    dialog.exec()
+                    if not dialog.result():
+                        return
+
+                    RS.settings.setValue('hide_jack_config_script_dialog',
+                                         dialog.notAgainValue())
+
+                    autostart_jack_checker = dialog.autostartValue()
+                    action = 'set_jack_checker_autostart'
+                    if not autostart_jack_checker:
+                        action = 'unset_jack_checker_autostart'
+
+                    self.toDaemon('/ray/server/exotic_action', action)
+
+            elif template_name == '///' + ray.factory_session_templates[2]:
+                if not RS.settings.value('hide_session_scripts_dialog',
+                                        False, type=bool):
+                    # display session scripts info dialog
+
+                    session_path = "%s/%s" % (CommandLineArgs.session_root,
+                                              session_short_path)
+
+                    dialog = child_dialogs.SessionScriptsInfoDialog(self,
+                                                                session_path)
+                    dialog.exec()
+                    if not dialog.result():
+                        return
+
+                    RS.settings.setValue('hide_session_scripts_dialog',
+                                         dialog.notAgainValue())
+
+        self.toDaemon('/ray/server/new_session', session_short_path,
+                      template_name)
 
     def openSession(self, action):
         dialog = child_dialogs.OpenSessionDialog(self)
@@ -415,9 +483,13 @@ class MainWindow(QMainWindow):
         if not dialog.result():
             return
 
-        RS.settings.setValue('last_session', self._session.name)
+        if self._session.isRunning():
+            # remember the running session as last session (if any)
+            short_path = self._session.getShortPath()
+            if not short_path.startswith('/'):
+                RS.settings.setValue('last_session', short_path)
 
-        session_name = dialog.getSessionName()
+        session_name = dialog.getSessionShortPath()
         self.toDaemon('/ray/session/duplicate', session_name)
 
     def saveTemplateSession(self):
@@ -428,16 +500,16 @@ class MainWindow(QMainWindow):
 
         session_template_name = dialog.getTemplateName()
         self.toDaemon('/ray/session/save_as_template', session_template_name)
-        
+
     def returnToAPreviousState(self):
         dialog = snapshots_dialog.SessionSnapshotsDialog(self)
         dialog.exec()
         if not dialog.result():
-            return 
-        
+            return
+
         snapshot = dialog.getSelectedSnapshot()
         self.toDaemon('/ray/session/open_snapshot', snapshot)
-    
+
     def aboutRaySession(self):
         dialog = child_dialogs.AboutRaySessionDialog(self)
         dialog.exec()
@@ -445,7 +517,7 @@ class MainWindow(QMainWindow):
     def donate(self, display_no_again=False):
         dialog = child_dialogs.DonationsDialog(self, display_no_again)
         dialog.exec()
-        
+
     def saveSession(self):
         self.toDaemon('/ray/session/save')
 
@@ -478,51 +550,46 @@ class MainWindow(QMainWindow):
         if not dialog.result():
             return
 
-        #command = dialog.getExecutableSelected()
-        #proxy = dialog.runViaProxy()
-        
-        command, via_proxy, prefix_mode, prefix, client_id = dialog.getSelection()
-        
-        self.toDaemon('/ray/session/add_executable', command, int(via_proxy),
-                      prefix_mode, prefix, client_id)
-        
-        #if proxy:
-            #self.toDaemon('/ray/session/add_proxy', command)
-        #else:
-            #self.toDaemon('/ray/session/add_executable', command)
+        command, auto_start, via_proxy, \
+            prefix_mode, prefix, client_id = dialog.getSelection()
+
+        self.toDaemon('/ray/session/add_executable', command, int(auto_start),
+                      int(via_proxy), prefix_mode, prefix, client_id)
 
     def stopClient(self, client_id):
         client = self._session.getClient(client_id)
         if not client:
             return
-        
-        if client.no_save_level and client.check_last_save:
-            dialog = child_dialogs.StopClientNoSaveDialog(self, client_id)
-            dialog.exec()
-            if not dialog.result():
-                return
 
-        elif (client.status == ray.ClientStatus.READY
-              and client.check_last_save):
-            if client.has_dirty:
-                if client.dirty_state:
+        if client.check_last_save:
+            if (client.no_save_level
+                    or (client.protocol == ray.Protocol.RAY_HACK
+                        and not client.ray_hack.saveable())):
+                dialog = child_dialogs.StopClientNoSaveDialog(self, client_id)
+                dialog.exec()
+                if not dialog.result():
+                    return
+
+            elif client.status == ray.ClientStatus.READY:
+                if client.has_dirty:
+                    if client.dirty_state:
+                        dialog = child_dialogs.StopClientDialog(self, client_id)
+                        dialog.exec()
+                        if not dialog.result():
+                            return
+
+                # last save (or start) more than 60 seconds ago
+                elif (time.time() - client.last_save) >= 60:
                     dialog = child_dialogs.StopClientDialog(self, client_id)
                     dialog.exec()
                     if not dialog.result():
                         return
 
-            # last save (or start) more than 60 seconds ago
-            elif (time.time() - client.last_save) >= 10:
-                dialog = child_dialogs.StopClientDialog(self, client_id)
-                dialog.exec()
-                if not dialog.result():
-                    return
-
         self.toDaemon('/ray/client/stop', client_id)
 
     def statusBarPressed(self):
         status = self._session.server_status
-        
+
         if status not in (
                 ray.ServerStatus.PRECOPY,
                 ray.ServerStatus.COPY,
@@ -530,7 +597,7 @@ class MainWindow(QMainWindow):
                 ray.ServerStatus.OUT_SNAPSHOT,
                 ray.ServerStatus.WAIT_USER):
             return
-        
+
         if status in (ray.ServerStatus.PRECOPY, ray.ServerStatus.COPY):
             if not self.server_copying:
                 return
@@ -542,11 +609,11 @@ class MainWindow(QMainWindow):
                 return
 
             self.toDaemon('/ray/server/abort_copy')
-        
+
         elif status in (ray.ServerStatus.SNAPSHOT,
                         ray.ServerStatus.OUT_SNAPSHOT):
             self.showSnapshotProgressDialog()
-            
+
         elif status == ray.ServerStatus.WAIT_USER:
             dialog = child_dialogs.WaitingCloseUserDialog(self)
             dialog.exec()
@@ -568,12 +635,12 @@ class MainWindow(QMainWindow):
 
         self.toDaemon('/ray/server/abort_copy')
 
-    def renameSession(self, new_session_name):
+    def renameSessionConditionnaly(self, new_session_name):
         self.toDaemon('/ray/session/rename', new_session_name)
-        
+
     def addFavorite(self, name, icon_name, factory):
         self._session.addFavorite(name, icon_name, factory, True)
-        
+
     def removeFavorite(self, name, factory):
         self._session.removeFavorite(name, factory, True)
 
@@ -581,18 +648,18 @@ class MainWindow(QMainWindow):
         if self.progress_dialog_visible:
             return
         self.progress_dialog_visible = True
-        
+
         dialog = child_dialogs.SnapShotProgressDialog(self)
         dialog.serverProgress(self.server_progress)
         dialog.exec()
-        
+
         self.progress_dialog_visible = False
-        
+
         if not dialog.result():
-            return 
-        
+            return
+
         self.toDaemon('/ray/server/abort_snapshot')
-        
+
     def showDaemonUrlWindow(self, err_code, ex_url=''):
         dialog = child_dialogs.DaemonUrlWindow(self, err_code, ex_url)
         dialog.exec()
@@ -613,6 +680,10 @@ class MainWindow(QMainWindow):
 
         self._signaler.daemon_url_changed.emit(new_url)
 
+    def clientPropertiesStateChanged(self, client_id, bool_visible):
+        self.ui.listWidget.clientPropertiesStateChanged(client_id,
+                                                        bool_visible)
+
     ###FUNCTIONS RELATED TO SIGNALS FROM OSC SERVER#######
 
     def removeClient(self, client_id):
@@ -655,7 +726,7 @@ class MainWindow(QMainWindow):
 
     def setSessionNameEditable(self, bool_set_edit):
         self.ui.stackedWidgetSessionName.setEditable(bool_set_edit)
-        
+
     def errorMessage(self, message):
         error_dialog = child_dialogs.ErrorDialog(self, message)
         error_dialog.exec()
@@ -682,7 +753,7 @@ class MainWindow(QMainWindow):
             serverStatusString(server_status))
         self.ui.frameCurrentSession.setEnabled(
             bool(server_status != ray.ServerStatus.OFF))
-        
+
         if server_status in (ray.ServerStatus.SNAPSHOT,
                              ray.ServerStatus.OUT_SNAPSHOT):
             self.timer_snapshot.start()
@@ -737,10 +808,10 @@ class MainWindow(QMainWindow):
 
         self.ui.stackedWidgetSessionName.setEditable(
             ready and self._session.is_renameable)
-        
+
         self.ui.trashButton.setEnabled(bool(self._session.trashed_clients)
                                        and not close_or_off)
-        
+
         if (CommandLineArgs.under_nsm
                 and not CommandLineArgs.out_daemon
                 and ready
@@ -757,17 +828,17 @@ class MainWindow(QMainWindow):
             self.ui.actionDuplicateSession.setEnabled(False)
             self.ui.actionCloseSession.setEnabled(False)
             self.ui.actionAbortSession.setEnabled(False)
-        
+
         if server_status == ray.ServerStatus.OFF:
             if self.terminate_request:
                 self._daemon_manager.stop()
-                
+
         if server_status == ray.ServerStatus.WAIT_USER:
             if not RS.settings.value(
                     'hide_wait_close_user_dialog', False, type=bool):
                 dialog = child_dialogs.WaitingCloseUserDialog(self)
                 dialog.exec()
-            
+
 
     def trashAdd(self, client_data):
         prettier_name = client_data.name
@@ -778,7 +849,7 @@ class MainWindow(QMainWindow):
             QIcon.fromTheme(client_data.icon), prettier_name)
         act_x_trashed.setData(client_data.client_id)
         act_x_trashed.triggered.connect(self.showClientTrashDialog)
-        
+
         self.ui.trashButton.setEnabled(
             bool(not self._session.server_status in (
                         ray.ServerStatus.OFF,
@@ -786,26 +857,26 @@ class MainWindow(QMainWindow):
                         ray.ServerStatus.WAIT_USER,
                         ray.ServerStatus.OUT_SNAPSHOT,
                         ray.ServerStatus.CLOSE)))
-        
+
         return act_x_trashed
-    
+
     def trashRemove(self, menu_action):
         self.trashMenu.removeAction(menu_action)
-        
+
         if not self._session.trashed_clients:
             self.ui.trashButton.setEnabled(False)
-    
+
     def trashClear(self):
         self.trashMenu.clear()
         self.ui.trashButton.setEnabled(False)
-    
+
     @pyqtSlot()
     def showClientTrashDialog(self):
         try:
             client_id = str(self.sender().data())
         except BaseException:
             return
-        
+
         for trashed_client in self._session.trashed_clients:
             if trashed_client.data.client_id == client_id:
                 break
@@ -817,40 +888,86 @@ class MainWindow(QMainWindow):
         if not dialog.result():
             return
 
-        self.toDaemon('/ray/trash/restore', client_id)
-    
+        self.toDaemon('/ray/trashed_client/restore', client_id)
+
     @pyqtSlot()
     def launchFavorite(self):
         template_name, factory = self.sender().data()
         self.toDaemon('/ray/session/add_client_template',
                       int(factory),
                       template_name)
-        
+
     def updateFavoritesMenu(self):
         self.favorites_menu.clear()
-        
+
         enable = bool(self._session.favorite_list
                       and not self._session.server_status in (
                         ray.ServerStatus.OFF,
                         ray.ServerStatus.CLOSE,
                         ray.ServerStatus.OUT_SAVE,
                         ray.ServerStatus.OUT_SNAPSHOT))
-        
+
         self.ui.toolButtonFavorites.setEnabled(enable)
-        
+
         for favorite in self._session.favorite_list:
             act_app = self.favorites_menu.addAction(
                         ray.getAppIcon(favorite.icon, self), favorite.name)
             act_app.setData([favorite.name, favorite.factory])
             act_app.triggered.connect(self.launchFavorite)
-        
+
+    def showScriptInfo(self, text):
+        if self.script_info_dialog and self.script_info_dialog.shouldBeRemoved():
+            del self.script_info_dialog
+            self.script_info_dialog = None
+
+        if not self.script_info_dialog:
+            self.script_info_dialog = child_dialogs.ScriptInfoDialog(self)
+
+        self.script_info_dialog.setInfoLabel(text)
+        self.script_info_dialog.show()
+
+    def hideScriptInfoDialog(self):
+        if self.script_info_dialog:
+            self.script_info_dialog.close()
+
+        del self.script_info_dialog
+        self.script_info_dialog = None
+
+    def showScriptUserActionDialog(self, text):
+        if self.script_action_dialog:
+            self.script_action_dialog.close()
+            del self.script_action_dialog
+            self.toDaemon('/error', '/ray/gui/script_user_action',
+                    ray.Err.NOT_NOW, 'another script_user_action take place')
+
+        self.script_action_dialog = child_dialogs.ScriptUserActionDialog(self)
+        self.script_action_dialog.setMainText(text)
+        self.script_action_dialog.show()
+
+
+    def hideScriptUserActionDialog(self):
+        if self.script_action_dialog:
+            self.script_action_dialog.close()
+            del self.script_action_dialog
+            self.script_action_dialog = None
+
+    def resizeWinWithMessages(self, messages_visible):
+        if messages_visible:
+            pass
+        else:
+            next_width = self.ui.frameCurrentSession.width()
+            next_height = self.height()
+            self.resize(next_width, next_height)
+            self.resizeEvent(None)
+            #self.setMaximumWidth(16777215)
+
     def daemonCrash(self):
         QMessageBox.critical(
             self, _translate(
                 'errors', "daemon crash!"), _translate(
                 'errors', "ray-daemon crashed, sorry !"))
         QApplication.quit()
-        
+
     def saveWindowSettings(self):
         RS.settings.setValue('MainWindow/geometry', self.saveGeometry())
         RS.settings.setValue('MainWindow/WindowState', self.saveState())
@@ -863,10 +980,10 @@ class MainWindow(QMainWindow):
         RS.settings.sync()
 
     # Reimplemented Functions
-    
+
     def closeEvent(self, event):
         self.saveWindowSettings()
-        
+
         if self.quitApp():
             QMainWindow.closeEvent(self, event)
         else:
